/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-11-28 16:40:55 +0900 #$
** $Rev: 2808 $
** $Author: sako@FITECHLABS.CO.JP $
**/

#include "common.h"

#include "log.h"
#include "db.h"
#include "dbcache.h"
#include "daemon.h"
#include "zbxserver.h"
#include "zbxself.h"
#include "../events.h"

#include "jacommon.h"
#include "jaloader.h"

#define JOBNET_LOAD_SPAN	"JOBNET_LOAD_SPAN"

#define VALID_FLAG_ON		1
#define VALID_FLAG_OFF		0

#define UPDATE_DATE_LEN		14 + 1
#define OPERATING_DATE_LEN	8  + 1
#define SCHEDULED_TIME_LEN	12 + 1

extern unsigned char	process_type;
extern int		process_num;

static int		no_valid_flag;
static char		msgwork[2048];


/******************************************************************************
 *                                                                            *
 * Function: get_search_date                                                  *
 *                                                                            *
 * Purpose: calculate the search time of the schedule                         *
 *                                                                            *
 * Parameters: span (in) - deployment time of a jobnet (minute)               *
 *             start_date (out) - search start time (YYYYMMDDHHMM)            *
 *             end_date (out) - search end time (current time) (YYYYMMDDHHMM) *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
void	get_search_date(int span, char **start_date, char **end_date)
{
	struct tm	*tm;
	time_t		now;
	const char	*__function_name = "get_search_date";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%d)", __function_name, span);

	time(&now);
	tm = localtime(&now);

	*end_date = zbx_dsprintf(NULL, "%04d%02d%02d%02d%02d",
				(tm->tm_year + 1900),
				(tm->tm_mon  + 1),
				 tm->tm_mday,
				 tm->tm_hour,
				 tm->tm_min);

	tm->tm_min = tm->tm_min + span;
	mktime(tm);

	*start_date = zbx_dsprintf(NULL, "%04d%02d%02d%02d%02d",
				(tm->tm_year + 1900),
				(tm->tm_mon  + 1),
				 tm->tm_mday,
				 tm->tm_hour,
				 tm->tm_min);

	return;
}

/******************************************************************************
 *                                                                            *
 * Function: get_load_span                                                    *
 *                                                                            *
 * Purpose: gets the deployment time of a jobnet                              *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value: deployment time jobnet                                       *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	get_load_span()
{
	DB_RESULT	result;
	DB_ROW		row;
	int		span = 60;
	const char	*__function_name = "get_load_span";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

	result = DBselect("select value from ja_parameter_table where parameter_name = '%s'", JOBNET_LOAD_SPAN);

	if (NULL != (row = DBfetch(result)))
	{
		span = atoi(row[0]);
		if (span < 1)
		{
			span = 1;
		}
		if (span > 1439)
		{
			span = 1439;
		}
	}
	DBfree_result(result);

	return span;
}

/******************************************************************************
 *                                                                            *
 * Function: make_end_operating_date                                          *
 *                                                                            *
 * Purpose: edit the end date search                                          *
 *                                                                            *
 * Parameters: end_ope_date (out) - search end time (YYYYMMDD)                *
 *             end_date (in) - search end time (current time) (YYYYMMDDHHMM)  *
 *             boot_time (in) - boot time (HHMM 0:00-24:00-nn:nn)             *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
void	make_end_operating_date(char *end_ope_date, char *end_date, char *boot_time)
{
	struct tm	tm;
	int		i_year, i_mon, i_day, i_bt, i_sday;
	char		year[5], mon[3], day[3];
	char		*edit_date;
	const char	*__function_name = "make_end_operating_date";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s)", __function_name, end_date, boot_time);

	i_bt = atoi(boot_time);
	if (i_bt < 2400)
	{
		zbx_strlcpy(end_ope_date, end_date, OPERATING_DATE_LEN);
		return;
	}

	zbx_strlcpy(year,  end_date,      5);
	zbx_strlcpy(mon,  (end_date + 4), 3);
	zbx_strlcpy(day,  (end_date + 6), 3);

	i_year = atoi(year);
	i_mon  = atoi(mon);
	i_day  = atoi(day);

	i_sday = i_bt / 2400;

	tm.tm_year  = i_year - 1900;
	tm.tm_mon   = i_mon - 1;
	tm.tm_mday  = i_day - i_sday;
	tm.tm_hour  = 0;
	tm.tm_min   = 0;
	tm.tm_sec   = 0;
	tm.tm_isdst = -1;

	mktime(&tm);

	edit_date = zbx_dsprintf(NULL, "%04d%02d%02d",
				(tm.tm_year + 1900),
				(tm.tm_mon  + 1),
				 tm.tm_mday);

	zbx_strlcpy(end_ope_date, edit_date, OPERATING_DATE_LEN);

	zbx_free(edit_date);
	return;
}

/******************************************************************************
 *                                                                            *
 * Function: make_scheduled_time                                              *
 *                                                                            *
 * Purpose: edit a scheduled time                                             *
 *                                                                            *
 * Parameters: scheduled_time (out) - time schedule that has been edited      *
 *                                    (YYYYMMDDHHMM)                          *
 *             operating_date (in) - operating date (YYYYMMDD)                *
 *             boot_time (in) - boot time (HHMM 0:00-24:00-)                  *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
void	make_scheduled_time(char *scheduled_time, char *operating_date, char *boot_time)
{
	struct tm	tm;
	int		i_year, i_mon, i_day, i_bt, i_sday, w_bt;
	char		year[5], mon[3], day[3];
	char		*edit_date;
	const char	*__function_name = "make_scheduled_time";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s)", __function_name, operating_date, boot_time);

	i_bt = atoi(boot_time);
	if (i_bt < 2400)
	{
		zbx_strlcpy(scheduled_time, operating_date, SCHEDULED_TIME_LEN);
		zbx_strlcpy((scheduled_time + 8), boot_time, 5);
		return;
	}

	zbx_strlcpy(year,  operating_date,      5);
	zbx_strlcpy(mon,  (operating_date + 4), 3);
	zbx_strlcpy(day,  (operating_date + 6), 3);

	i_year = atoi(year);
	i_mon  = atoi(mon);
	i_day  = atoi(day);

	i_sday = i_bt / 2400;
	w_bt   = i_bt - (2400 * i_sday);

	tm.tm_year  = i_year - 1900;
	tm.tm_mon   = i_mon - 1;
	tm.tm_mday  = i_day + i_sday;
	tm.tm_hour  = 0;
	tm.tm_min   = 0;
	tm.tm_sec   = 0;
	tm.tm_isdst = -1;

	mktime(&tm);

	edit_date = zbx_dsprintf(NULL, "%04d%02d%02d%04d",
				(tm.tm_year + 1900),
				(tm.tm_mon  + 1),
				 tm.tm_mday,
				 w_bt);

	zbx_strlcpy(scheduled_time, edit_date, 13);

	zbx_free(edit_date);
	return;
}

/******************************************************************************
 *                                                                            *
 * Function: already_start_check_immediate                                    *
 *                                                                            *
 * Purpose: check if the immediate start jobnet is already deployed           *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *                                                                            *
 * Return value: SUCCEED - not started                                        *
 *               FAIL - already started                                       *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	already_start_check_immediate(zbx_uint64_t inner_jobnet_id)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		count;
	const char	*__function_name = "already_start_check_immediate";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 ")", __function_name, inner_jobnet_id);

	result = DBselect("select count(*) from ja_run_jobnet_summary_table"
			" where inner_jobnet_id = " ZBX_FS_UI64, inner_jobnet_id);

	if (NULL == (row = DBfetch(result)))
	{
		zbx_snprintf(msgwork, sizeof(msgwork), ZBX_FS_UI64, inner_jobnet_id);
		ja_log("JALOADER200001", inner_jobnet_id, NULL, "ja_run_jobnet_summary_table", msgwork);
		DBfree_result(result);
		return FAIL;
	}

	count = atoi(row[0]);
	DBfree_result(result);

	if (count > 0)
	{
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: get_flow_inner_job_id                                            *
 *                                                                            *
 * Purpose: gets the internal job id for the job flow                         *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             job_id (in) - job id                                           *
 *                                                                            *
 * Return value: internal job id                                              *
 *               FAIL - an error occurred                                     *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static zbx_uint64_t	get_flow_inner_job_id(zbx_uint64_t inner_jobnet_id, char *job_id)
{
	DB_RESULT	result;
	DB_ROW		row;
	zbx_uint64_t	inner_job_id;
	const char	*__function_name = "get_flow_inner_job_id";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " %s)",
		__function_name, inner_jobnet_id, job_id);

	result = DBselect("select inner_job_id from ja_run_job_table"
			" where inner_jobnet_id = " ZBX_FS_UI64 " and job_id = '%s'",
			inner_jobnet_id, job_id);

	if (NULL == (row = DBfetch(result)))
	{
		zbx_snprintf(msgwork, sizeof(msgwork), ZBX_FS_UI64 " %s", inner_jobnet_id, job_id);
		ja_log("JALOADER200001", inner_jobnet_id, NULL, "ja_run_job_table", msgwork);
		DBfree_result(result);
		return FAIL;
	}

	ZBX_STR2UINT64(inner_job_id, row[0]);

	DBfree_result(result);

	return inner_job_id;
}

/******************************************************************************
 *                                                                            *
 * Function: get_boot_count                                                   *
 *                                                                            *
 * Purpose: gets the total number of flow                                     *
 *                                                                            *
 * Parameters: jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value: number of flows                                              *
 *               FAIL - an error occurred                                     *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	get_boot_count(char *jobnet_id, char *job_id, char *update_date)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		count;
	const char	*__function_name = "get_boot_count";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s %s)", __function_name, jobnet_id, job_id, update_date);

	result = DBselect("select count(*) from ja_flow_control_table"
			" where jobnet_id = '%s' and end_job_id = '%s' and update_date = %s",
			jobnet_id, job_id, update_date);

	if (NULL == (row = DBfetch(result)))
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200001", 0, jobnet_id, "ja_flow_control_table", msgwork);
		DBfree_result(result);
		return FAIL;
	}

	count = atoi(row[0]);

	DBfree_result(result);

	return count;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_end                                                    *
 *                                                                            *
 * Purpose: end icon to expand the information                                *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_end(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		rc, jobnet_stop_flag;
	const char	*__function_name = "load_icon_end";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_end_table ("
			" inner_job_id, inner_jobnet_id, jobnet_stop_flag, jobnet_stop_code)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', jobnet_stop_flag, jobnet_stop_code"
			" from ja_icon_end_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_end_table", msgwork);
		return FAIL;
	}

	/* get jobnet stop flag */
	result = DBselect("select jobnet_stop_flag from ja_icon_end_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			jobnet_id, job_id, update_date);

	if (NULL == (row = DBfetch(result)))
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200001", 0, jobnet_id, "ja_icon_end_table", msgwork);
		DBfree_result(result);
		return FAIL;
	}

	jobnet_stop_flag = atoi(row[0]);
	if (jobnet_stop_flag == 1)
	{
		rc = DBexecute("update ja_run_job_table set method_flag = %d where inner_job_id = " ZBX_FS_UI64,
				JA_JOB_METHOD_WAIT, inner_job_id);

		if (rc <= ZBX_DB_OK)
		{
			zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s " ZBX_FS_UI64, row[0], row[1], row[2],
				inner_job_id);
			ja_log("JALOADER200002", 0, jobnet_id, "ja_run_job_table", msgwork);
			DBfree_result(result);
			return FAIL;
		}
	}

	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_if                                                     *
 *                                                                            *
 * Purpose: if icon to expand the information                                 *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_if(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_if";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_if_table ("
			" inner_job_id, inner_jobnet_id, hand_flag, value_name, comparison_value)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', hand_flag, value_name, comparison_value"
			" from ja_icon_if_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_icon_if_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_value                                                  *
 *                                                                            *
 * Purpose: job controller variable icon to expand the information            *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_value(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_value";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_value_table ("
			" inner_job_id, inner_jobnet_id, value_name, value)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', value_name, value"
			" from ja_icon_value_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_value_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_job                                                    *
 *                                                                            *
 * Purpose: job icon to expand the information                                *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_job(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_job";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);


	/* ja_run_icon_job_table additional data */
	rc = DBexecute("insert into ja_run_icon_job_table ("
			" inner_job_id, inner_jobnet_id, host_flag, stop_flag,"
			" command_type, timeout, host_name, stop_code)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', host_flag, stop_flag,"
			" command_type, timeout, host_name, stop_code"
			" from ja_icon_job_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_job_table", msgwork);
		return FAIL;
	}


	/* ja_run_job_command_table additional data */
	rc = DBexecute("insert into ja_run_job_command_table ("
			" inner_job_id, inner_jobnet_id,"
			" command_cls, command)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', command_cls, command"
			" from ja_job_command_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_job_command_table", msgwork);
		return FAIL;
	}


	/* ja_run_value_job_table additional data */
	rc = DBexecute("insert into ja_run_value_job_table ("
			" inner_job_id, inner_jobnet_id,"
			" value_name, value)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', value_name, value"
			" from ja_value_job_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc < ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_value_job_table", msgwork);
		return FAIL;
	}


	/* ja_run_value_jobcon_table additional data */
	rc = DBexecute("insert into ja_run_value_jobcon_table ("
			" inner_job_id, inner_jobnet_id, value_name)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', value_name"
			" from ja_value_jobcon_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc < ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_value_jobcon_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_jobnet                                                 *
 *                                                                            *
 * Purpose: jobnet icon to expand the information                             *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_jobnet(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	zbx_uint64_t	link_inner_jobnet_id;
	const char	*__function_name = "load_icon_jobnet";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	link_inner_jobnet_id = get_next_id(JA_RUN_ID_JOBNET_LD, inner_jobnet_id, jobnet_id);
	if (link_inner_jobnet_id == 0)
	{
		return FAIL;
	}

	rc = DBexecute("insert into ja_run_icon_jobnet_table ("
			" inner_job_id, inner_jobnet_id, link_inner_jobnet_id, link_jobnet_id)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', link_jobnet_id"
			" from ja_icon_jobnet_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, link_inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_jobnet_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_extjob                                                 *
 *                                                                            *
 * Purpose: extended job icon to expand the information                       *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_extjob(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_extjob";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_extjob_table ("
			" inner_job_id, inner_jobnet_id, command_id, value, pid)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', command_id, value, 0"
			" from ja_icon_extjob_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_extjob_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_calc                                                   *
 *                                                                            *
 * Purpose: calculation icon to expand the information                        *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_calc(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_calc";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_calc_table ("
			" inner_job_id, inner_jobnet_id, hand_flag, formula, value_name)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', hand_flag, formula, value_name"
			" from ja_icon_calc_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_calc_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_task                                                   *
 *                                                                            *
 * Purpose: task icon to expand the information                               *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_task(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	zbx_uint64_t	submit_inner_jobnet_id;
	const char	*__function_name = "load_icon_task";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	submit_inner_jobnet_id = get_next_id(JA_RUN_ID_JOBNET_LD, inner_jobnet_id, jobnet_id);
	if (submit_inner_jobnet_id == 0)
	{
		return FAIL;
	}

	rc = DBexecute("insert into ja_run_icon_task_table ("
			" inner_job_id, inner_jobnet_id, submit_inner_jobnet_id, submit_jobnet_id)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', submit_jobnet_id"
			" from ja_icon_task_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, submit_inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_task_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_icon_info                                                   *
 *                                                                            *
 * Purpose: calculation icon to expand the information                        *
 *                                                                            *
 * Parameters: inner_jobnet_id (in) - inner jobnet id                         *
 *             inner_job_id (in) - inner job id                               *
 *             jobnet_id (in) - jobnet id                                     *
 *             job_id (in) - job id                                           *
 *             update_date (in) - update date                                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_icon_info(zbx_uint64_t inner_jobnet_id, zbx_uint64_t inner_job_id,
				char *jobnet_id, char *job_id, char *update_date)
{
	int		rc;
	const char	*__function_name = "load_icon_info";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %s)",
		__function_name, inner_jobnet_id, inner_job_id, jobnet_id, job_id, update_date);

	rc = DBexecute("insert into ja_run_icon_info_table ("
			" inner_job_id, inner_jobnet_id, info_flag, item_id,"
			" trigger_id, host_group, host_name, get_job_id)"
			" select '" ZBX_FS_UI64 "','" ZBX_FS_UI64 "', info_flag, item_id,"
			" trigger_id, host_group, host_name, get_job_id"
			" from ja_icon_info_table"
			" where jobnet_id = '%s' and job_id = '%s' and update_date = %s",
			inner_job_id, inner_jobnet_id, jobnet_id, job_id, update_date);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", jobnet_id, job_id, update_date);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_icon_info_table", msgwork);
		return FAIL;
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_job_definition                                              *
 *                                                                            *
 * Purpose: expand the job definition                                         *
 *                                                                            *
 * Parameters: inner_jobnet_main_id (in) - inner jobnet main id               *
 *             inner_jobnet_id (in) - inner jobnet id                         *
 *             jobnet_id (in) - jobnet id                                     *
 *             update_date (in) - update date                                 *
 *             test_flag (in) - test flag                                     *
 *             run_type (in) - run type                                       *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_job_definition(zbx_uint64_t inner_jobnet_main_id, zbx_uint64_t inner_jobnet_id,
					char *jobnet_id, char *update_date, int test_flag, int run_type)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		rc, job_type, boot_count;
	char		*job_name;
	zbx_uint64_t	inner_job_id, inner_job_id_fs_link, inner_flow_id, start_inner_job_id, end_inner_job_id;
	const char	*__function_name = "load_job_definition";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " " ZBX_FS_UI64 " %s %s %d %d)",
		__function_name, inner_jobnet_main_id, inner_jobnet_id, jobnet_id, update_date,
		test_flag, run_type);

	/* expand the information icon */
	result = DBselect("select jobnet_id, job_id, update_date, job_type, point_x, point_y, job_name"
			" from ja_job_control_table"
			" where jobnet_id = '%s' and update_date = %s",
			jobnet_id, update_date);

	while (NULL != (row = DBfetch(result)))
	{
		job_type = atoi(row[3]);

		inner_job_id = get_next_id(JA_RUN_ID_JOB, inner_jobnet_main_id, jobnet_id);
		if (inner_job_id == 0)
		{
			DBfree_result(result);
			return FAIL;
		}

		inner_job_id_fs_link = 0;
		if (job_type == JA_JOB_TYPE_JOB)
		{
			inner_job_id_fs_link = get_next_id(JA_RUN_ID_JOB, inner_jobnet_main_id, jobnet_id);
			if (inner_job_id_fs_link == 0)
			{
				DBfree_result(result);
				return FAIL;
			}
		}

		boot_count = get_boot_count(row[0], row[1], row[2]);
		if (boot_count == FAIL)
		{
			DBfree_result(result);
			return FAIL;
		}

		if (SUCCEED != DBis_null(row[6]))
		{
			job_name = row[6];
		}
		else
		{
			job_name = "";
		}

		rc = DBexecute("insert into ja_run_job_table ("
				" inner_job_id, inner_jobnet_id, inner_jobnet_main_id, inner_job_id_fs_link,"
				" invo_flag, job_type, test_flag, method_flag, timeout_flag, status,"
				" boot_count, end_count, start_time, end_time, point_x, point_y, job_id, job_name)"
				" values (" ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 ","
				" %d, %d, %d, %d, 0, %d,"
				" %d, 0, 0, 0, %s, %s, '%s', '%s')",
				inner_job_id, inner_jobnet_id, inner_jobnet_main_id, inner_job_id_fs_link,
				JA_JOB_INVO_FLAG_OFF, job_type, test_flag, JA_JOB_METHOD_NORMAL, JA_JOB_STATUS_BEGIN,
				boot_count, row[4], row[5], row[1], job_name);

		if (rc <= ZBX_DB_OK)
		{
			zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s", row[0], row[1], row[2]);
			ja_log("JALOADER200002", 0, jobnet_id, "ja_run_job_table", msgwork);
			DBfree_result(result);
			return FAIL;
		}

		rc = SUCCEED;
		switch (job_type)
		{
			case JA_JOB_TYPE_START:
				if (run_type == JA_JOBNET_RUN_TYPE_WAIT)
				{
					rc = DBexecute("update ja_run_job_table set method_flag = %d where inner_job_id = " ZBX_FS_UI64,
							JA_JOB_METHOD_WAIT, inner_job_id);

					if (rc <= ZBX_DB_OK)
					{
						zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s " ZBX_FS_UI64, row[0], row[1], row[2],
							inner_job_id);
						ja_log("JALOADER200002", 0, jobnet_id, "ja_run_job_table", msgwork);
						DBfree_result(result);
						return FAIL;
					}
					rc = SUCCEED;
				}
				break;

			case JA_JOB_TYPE_END:
				rc = load_icon_end(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_IF:
				rc = load_icon_if(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_VALUE:
				rc = load_icon_value(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_JOB:
				rc = load_icon_job(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_JOBNET:
				rc = load_icon_jobnet(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_EXTJOB:
				rc = load_icon_extjob(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_CALC:
				rc = load_icon_calc(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_TASK:
				rc = load_icon_task(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;

			case JA_JOB_TYPE_INFO:
				rc = load_icon_info(inner_jobnet_id, inner_job_id, row[0], row[1], row[2]);
				break;
		}

		if (rc != SUCCEED)
		{
			DBfree_result(result);
			return FAIL;
		}
	}

	DBfree_result(result);


	/* expand the information flow */
	result = DBselect("select start_job_id, end_job_id, flow_type, flow_width"
			" from ja_flow_control_table"
			" where jobnet_id = '%s' and update_date = %s",
			jobnet_id, update_date);

	while (NULL != (row = DBfetch(result)))
	{
		inner_flow_id = get_next_id(JA_RUN_ID_FLOW, inner_jobnet_main_id, jobnet_id);
		if (inner_flow_id == 0)
		{
			DBfree_result(result);
			return FAIL;
		}

		start_inner_job_id = get_flow_inner_job_id(inner_jobnet_id, row[0]);
		if (start_inner_job_id == FAIL)
		{
			DBfree_result(result);
			return FAIL;
		}

		end_inner_job_id = get_flow_inner_job_id(inner_jobnet_id, row[1]);
		if (end_inner_job_id == FAIL)
		{
			DBfree_result(result);
			return FAIL;
		}

		rc = DBexecute("insert into ja_run_flow_table ("
				" inner_flow_id, inner_jobnet_id, start_inner_job_id, end_inner_job_id,"
				" flow_type, flow_width)"
				" values (" ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 ","
				" %s, %s)",
				inner_flow_id, inner_jobnet_id, start_inner_job_id, end_inner_job_id,
				row[2], row[3]);

		if (rc <= ZBX_DB_OK)
		{
			zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %s %s", jobnet_id, update_date, row[0], row[1]);
			ja_log("JALOADER200002", 0, jobnet_id, "ja_run_flow_table", msgwork);
			DBfree_result(result);
			return FAIL;
		}
	}

	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_sub_jobnet_definition                                       *
 *                                                                            *
 * Purpose: expand the sub jobnet definition                                  *
 *                                                                            *
 * Parameters: inner_jobnet_main_id (in) - inner jobnet main id               *
 *             run_type (in) - run type                                       *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_sub_jobnet_definition(zbx_uint64_t inner_jobnet_main_id, int run_type)
{
	DB_RESULT	result;
	DB_RESULT	result2;
	DB_RESULT	result3;
	DB_ROW		row;
	DB_ROW		row2;
	DB_ROW		row3;
	int		hit, test_flag, rc;
	zbx_uint64_t	link_inner_jobnet_id;
	char		*memo;
	const char	*__function_name = "load_sub_jobnet_definition";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(" ZBX_FS_UI64 " %d)",
		__function_name, inner_jobnet_main_id, run_type);

	hit = 1;
	while (hit == 1)
	{
		hit = 0;
		result = DBselect("select inner_job_id from ja_run_job_table"
				" where inner_jobnet_main_id = " ZBX_FS_UI64 " and job_type = %d and invo_flag = %d",
				inner_jobnet_main_id, JA_JOB_TYPE_JOBNET, JA_JOB_INVO_FLAG_OFF);

		while (NULL != (row = DBfetch(result)))
		{
			hit = 1;
			result2 = DBselect("select link_inner_jobnet_id, link_jobnet_id"
					" from ja_run_icon_jobnet_table"
					" where inner_job_id = %s", row[0]);

			if (NULL == (row2 = DBfetch(result2)))
			{
				ja_log("JALOADER200001", inner_jobnet_main_id, NULL, "ja_run_icon_jobnet_table", row[0]);
				DBfree_result(result);
				DBfree_result(result2);
				return FAIL;
			}

			ZBX_STR2UINT64(link_inner_jobnet_id, row2[0]);

			/* expand the sub jobnet */
			result3 = DBselect("select jobnet_id, update_date, public_flag, user_name, jobnet_name, memo"
					" from ja_jobnet_control_table"
					" where jobnet_id = '%s' and valid_flag = %d",
					row2[1], VALID_FLAG_ON);

			if (NULL == (row3 = DBfetch(result3)))
			{
				no_valid_flag = 1;
				ja_log("JALOADER200005", inner_jobnet_main_id, NULL, row2[1]);
				DBfree_result(result);
				DBfree_result(result2);
				DBfree_result(result3);
				return FAIL;
			}

			if (SUCCEED != DBis_null(row3[5]))
			{
				memo = row3[5];
			}
			else
			{
				memo = "";
			}

			rc = DBexecute("insert into ja_run_jobnet_table ("
					" inner_jobnet_id, inner_jobnet_main_id, inner_job_id, update_date, run_type, main_flag,"
					" timeout_flag, status, scheduled_time, start_time, end_time, public_flag,"
					" jobnet_id, user_name, jobnet_name, memo)"
					" values (" ZBX_FS_UI64 ", " ZBX_FS_UI64 ", %s, %s, %d, %d,"
					" 0, %d, 0, 0, 0, %s,"
					" '%s', '%s', '%s', '%s')",
					link_inner_jobnet_id, inner_jobnet_main_id, row[0], row3[1], run_type, JA_JOBNET_MAIN_FLAG_SUB,
					JA_JOBNET_STATUS_BEGIN, row3[2],
					row3[0], row3[3], row3[4], memo);

			if (rc <= ZBX_DB_OK)
			{
				zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row3[0], row3[1]);
				ja_log("JALOADER200002", inner_jobnet_main_id, NULL, "ja_run_jobnet_table", msgwork);
				DBfree_result(result);
				DBfree_result(result2);
				DBfree_result(result3);
				return FAIL;
			}

			if (run_type == JA_JOBNET_RUN_TYPE_TEST)
			{
				test_flag = JA_JOB_TEST_FLAG_ON;
			}
			else
			{
				test_flag = JA_JOB_TEST_FLAG_OFF;
			}

			/* expand the sub job */
			rc = load_job_definition(inner_jobnet_main_id, link_inner_jobnet_id, row3[0], row3[1],
					test_flag, JA_JOBNET_RUN_TYPE_NORMAL);
			if (rc != SUCCEED)
			{
				DBfree_result(result);
				DBfree_result(result2);
				DBfree_result(result3);
				return FAIL;
			}

			rc = DBexecute("update ja_run_job_table set invo_flag = %d where inner_job_id = %s", JA_JOB_INVO_FLAG_ON, row[0]);
			if (rc <= ZBX_DB_OK)
			{
				ja_log("JALOADER200003", inner_jobnet_main_id, NULL, "ja_run_job_table", row[0]);
				DBfree_result(result);
				DBfree_result(result2);
				DBfree_result(result3);
				return FAIL;
			}
			DBfree_result(result2);
			DBfree_result(result3);
		}
		DBfree_result(result);
	}

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: load_jobnet_definition                                           *
 *                                                                            *
 * Purpose: expand the jobnet definition                                      *
 *                                                                            *
 * Parameters: jobnet_id (in) - jobnet id                                     *
 *             scheduled_time (in) - scheduled time                           *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	load_jobnet_definition(char *jobnet_id, char *scheduled_time)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		count = 0, rc;
	zbx_uint64_t	inner_jobnet_main_id;
	char		*memo;
	const char	*__function_name = "load_jobnet_definition";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s)",
		__function_name, jobnet_id, scheduled_time);

	/* double check the expansion of jobnet */
	result = DBselect("select count(*)"
			" from ja_run_jobnet_table"
			" where jobnet_id = '%s' and scheduled_time = %s and run_type = %d and main_flag = %d",
			jobnet_id, scheduled_time, JA_JOBNET_RUN_TYPE_NORMAL, JA_JOBNET_MAIN_FLAG_MAIN);

	if (NULL == (row = DBfetch(result)))
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s %d %d",
			jobnet_id, scheduled_time, JA_JOBNET_RUN_TYPE_NORMAL, JA_JOBNET_MAIN_FLAG_MAIN);
		ja_log("JALOADER200001", 0, jobnet_id, "ja_run_jobnet_table", msgwork);
		DBfree_result(result);
		return FAIL;
	}

	count = atoi(row[0]);
	DBfree_result(result);

	/* already registered ? */
	if (count > 0)
	{
		/* skip registration */
		zabbix_log(LOG_LEVEL_DEBUG, "-DEBUG- jobnet already registered: jobnet[%s] scheduled time[%s]", jobnet_id, scheduled_time);
		return SUCCEED;
	}

	DBbegin();

	inner_jobnet_main_id = get_next_id(JA_RUN_ID_JOBNET_LD, 0, jobnet_id);
	if (inner_jobnet_main_id == 0)
	{
		DBrollback();
		return FAIL;
	}

	/* expand the main jobnet */
	result = DBselect("select jobnet_id, update_date, public_flag, user_name, jobnet_name, memo"
			" from ja_jobnet_control_table"
			" where jobnet_id = '%s' and valid_flag = %d",
			jobnet_id, VALID_FLAG_ON);

	if (NULL == (row = DBfetch(result)))
	{
		no_valid_flag = 1;
		ja_log("JALOADER200004", 0, NULL, jobnet_id);
		DBfree_result(result);
		DBrollback();
		return FAIL;
	}

	if (SUCCEED != DBis_null(row[5]))
	{
		memo = row[5];
	}
	else
	{
		memo = "";
	}

	rc = DBexecute("insert into ja_run_jobnet_table ("
			" inner_jobnet_id, inner_jobnet_main_id, inner_job_id, update_date,"
			" run_type, main_flag, timeout_flag, status,"
			" scheduled_time, start_time, end_time, public_flag,"
			" jobnet_id, user_name, jobnet_name, memo)"
			" values (" ZBX_FS_UI64 ", " ZBX_FS_UI64 ", 0, %s,"
			" %d, %d, 0, %d,"
			" %s, 0, 0, %s,"
			" '%s', '%s', '%s', '%s')",
			inner_jobnet_main_id, inner_jobnet_main_id, row[1],
			JA_JOBNET_RUN_TYPE_NORMAL, JA_JOBNET_MAIN_FLAG_MAIN, JA_JOBNET_STATUS_BEGIN,
			scheduled_time, row[2],
			row[0], row[3], row[4], memo);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row[0], row[1]);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_jobnet_table", msgwork);
		DBfree_result(result);
		DBrollback();
		return FAIL;
	}

	rc = DBexecute("insert into ja_run_jobnet_summary_table ("
			" inner_jobnet_id, update_date, invo_flag, run_type, status, job_status, jobnet_abort_flag,"
			" scheduled_time, start_time, end_time, public_flag, jobnet_id, user_name, jobnet_name, memo)"
			" values (" ZBX_FS_UI64 ", %s, 1, %d, %d, 0, 0,"
			" %s, 0, 0, %s, '%s', '%s', '%s', '%s')",
			inner_jobnet_main_id, row[1], JA_JOBNET_RUN_TYPE_NORMAL, JA_JOBNET_STATUS_BEGIN,
			scheduled_time, row[2], row[0], row[3], row[4], memo);

	if (rc <= ZBX_DB_OK)
	{
		zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row[0], row[1]);
		ja_log("JALOADER200002", 0, jobnet_id, "ja_run_jobnet_summary_table", msgwork);
		DBfree_result(result);
		DBrollback();
		return FAIL;
	}

	/* expand the main job */
	rc = load_job_definition(inner_jobnet_main_id, inner_jobnet_main_id, row[0], row[1],
			JA_JOB_TEST_FLAG_OFF, JA_JOBNET_RUN_TYPE_NORMAL);
	if (rc != SUCCEED)
	{
		DBfree_result(result);
		DBrollback();
		return FAIL;
	}

	/* expand the sub jobnet */
	rc = load_sub_jobnet_definition(inner_jobnet_main_id, JA_JOBNET_RUN_TYPE_NORMAL);
	if (rc != SUCCEED)
	{
		DBfree_result(result);
		DBrollback();
		return FAIL;
	}

	DBfree_result(result);
	DBcommit();
	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: get_schedule_jobnet                                              *
 *                                                                            *
 * Purpose: to get the jobnet for execution                                   *
 *                                                                            *
 * Parameters: schedule_id (in) - schedule id                                 *
 *             update_date (in) - update date                                 *
 *             scheduled_time (in) - scheduled time                           *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	get_schedule_jobnet(char *schedule_id, char *update_date, char *scheduled_time)
{
	DB_RESULT	result;
	DB_ROW		row;
	const char	*__function_name = "get_schedule_jobnet";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s %s)",
		__function_name, schedule_id, update_date, scheduled_time);

	result = DBselect("select jobnet_id"
			" from ja_schedule_jobnet_table"
			" where schedule_id = '%s' and update_date = %s",
			schedule_id, update_date);

	while (NULL != (row = DBfetch(result)))
	{
		load_jobnet_definition(row[0], scheduled_time);
	}
	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: get_calendar_detail                                              *
 *                                                                            *
 * Purpose: check the working days of jobnet                                  *
 *                                                                            *
 * Parameters: schedule_id (in) - schedule id                                 *
 *             update_date (in) - update date                                 *
 *             start_date (in) - search start time                            *
 *             end_date (in) - search end time (current time)                 *
 *             calendar_id (in) - calendar id                                 *
 *             boot_time (in) - boot time                                     *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	get_calendar_detail(char *schedule_id, char *update_date,
				char *start_date, char *end_date, char *calendar_id , char *boot_time)
{
	DB_RESULT	result;
	DB_ROW		row;
	char		jc_update_date[UPDATE_DATE_LEN];
	char		start_ope_date[OPERATING_DATE_LEN], end_ope_date[OPERATING_DATE_LEN];
	char		scheduled_time[SCHEDULED_TIME_LEN];
	const char	*__function_name = "get_calendar_detail";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s %s %s %s %s)",
		__function_name, schedule_id, update_date, start_date, end_date, calendar_id, boot_time);

	result = DBselect("select update_date"
			" from ja_calendar_control_table"
			" where calendar_id = '%s' and valid_flag = %d",
			calendar_id, VALID_FLAG_ON);

	if (NULL == (row = DBfetch(result)))
	{
		no_valid_flag = 1;
		ja_log("JALOADER200006", 0, NULL, calendar_id);
		DBfree_result(result);
		return FAIL;
	}

	zbx_strlcpy(jc_update_date, row[0], sizeof(jc_update_date));
	DBfree_result(result);

	/* edit the start and end dates of the search range */
	zbx_strlcpy(start_ope_date, start_date, sizeof(start_ope_date));
	make_end_operating_date(end_ope_date, end_date, boot_time);

	result = DBselect("select operating_date"
			" from ja_calendar_detail_table"
			" where calendar_id = '%s' and update_date = %s"
			" and operating_date <= %s and operating_date >= %s",
			calendar_id, jc_update_date, start_ope_date, end_ope_date);

	while (NULL != (row = DBfetch(result)))
	{
		/* schedule time edit and check */
		make_scheduled_time(scheduled_time, row[0], boot_time);
		zabbix_log(LOG_LEVEL_DEBUG, "-DEBUG- scheduled_time[%s] <= start_date[%s] && scheduled_time[%s] >= end_date[%s]",
			scheduled_time, start_date, scheduled_time, end_date);
		if (strcmp(scheduled_time, start_date) <= 0 && strcmp(scheduled_time, end_date) >= 0)
		{
			get_schedule_jobnet(schedule_id, update_date, scheduled_time);
		}
	}
	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: get_schedule_detail                                              *
 *                                                                            *
 * Purpose: to get more information jobnet                                    *
 *                                                                            *
 * Parameters: schedule_id (in) - schedule id                                 *
 *             update_date (in) - update date                                 *
 *             start_date (in) - search start time                            *
 *             end_date (in) - search end time (current time)                 *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	get_schedule_detail(char *schedule_id, char *update_date, char *start_date, char *end_date)
{
	DB_RESULT	result;
	DB_ROW		row;
	const char	*__function_name = "get_schedule_detail";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%s %s %s %s)",
		__function_name, schedule_id, update_date, start_date, end_date);

	result = DBselect("select calendar_id, boot_time"
			" from ja_schedule_detail_table"
			" where schedule_id = '%s' and update_date = %s",
			schedule_id, update_date);

	while (NULL != (row = DBfetch(result)))
	{
		get_calendar_detail(schedule_id, update_date, start_date, end_date, row[0], row[1]);
	}
	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: jobnet_load_schedule                                             *
 *                                                                            *
 * Purpose: expand the jobnet (scheduled start)                               *
 *                                                                            *
 * Parameters: load_span (in) - deployment time of a jobnet                   *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	jobnet_load_schedule(int load_span)
{
	DB_RESULT	result;
	DB_ROW		row;
	int		rc;
	char		*start_date, *end_date;
	const char	*__function_name = "jobnet_load_schedule";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s(%d)", __function_name, load_span);

	get_search_date(load_span, &start_date, &end_date);

	result = DBselect("select schedule_id, update_date from ja_schedule_control_table where valid_flag = %d", VALID_FLAG_ON);

	while (NULL != (row = DBfetch(result)))
	{
		no_valid_flag = 0;

		get_schedule_detail(row[0], row[1], start_date, end_date);

		if (no_valid_flag != 0)
		{
			DBbegin();
			rc = DBexecute("update ja_schedule_control_table set valid_flag = %d"
					" where schedule_id = '%s' and update_date = %s",
					 VALID_FLAG_OFF, row[0], row[1]);

			if (rc <= ZBX_DB_OK)
			{
				zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row[0], row[1]);
				ja_log("JALOADER200003", 0, NULL, "ja_schedule_control_table", msgwork);
				DBrollback();
				continue;
			}
			DBcommit();
		}
	}
	DBfree_result(result);
	zbx_free(start_date);
	zbx_free(end_date);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: jobnet_load_immediate                                            *
 *                                                                            *
 * Purpose: expand the immediate start jobnet                                 *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:  SUCCEED - processed successfully                            *
 *                FAIL - an error occurred                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
static int	jobnet_load_immediate()
{
	DB_RESULT	result;
	DB_ROW		row;
	struct tm	*tm;
	time_t		now;
	int		run_type, test_flag, rc;
	zbx_uint64_t	inner_jobnet_id;
	char		*memo;
	char		now_time[20];
	const char	*__function_name = "jobnet_load_immediate";

	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);

	result = DBselect("select inner_jobnet_id, inner_job_id, update_date, run_type, scheduled_time,"
			" public_flag, jobnet_id, user_name, jobnet_name, memo"
			" from ja_run_jobnet_table"
			" where run_type <> %d and main_flag = %d and status = %d",
			JA_JOBNET_RUN_TYPE_NORMAL, JA_JOBNET_MAIN_FLAG_MAIN, JA_JOBNET_STATUS_BEGIN);

	while (NULL != (row = DBfetch(result)))
	{
		ZBX_STR2UINT64(inner_jobnet_id, row[0]);

		rc = already_start_check_immediate(inner_jobnet_id);
		if (rc != SUCCEED)
		{
			/* skip registration */
			zabbix_log(LOG_LEVEL_DEBUG, "-DEBUG- immediate jobnet already registered: jobnet[%s] inner_jobnet_id[%s]", row[6], row[0]);
			continue;
		}

		DBbegin();

		no_valid_flag = 0;

		if (SUCCEED != DBis_null(row[9]))
		{
			memo = row[9];
		}
		else
		{
			memo = "";
		}

		run_type = atoi(row[3]);
		if (run_type == JA_JOBNET_RUN_TYPE_TEST)
		{
			test_flag = JA_JOB_TEST_FLAG_ON;
		}
		else
		{
			test_flag = JA_JOB_TEST_FLAG_OFF;
		}

		/* expand the main job */
		rc = load_job_definition(inner_jobnet_id, inner_jobnet_id, row[6], row[2], test_flag, run_type);
		if (rc != SUCCEED)
		{
			DBrollback();
			continue;
		}

		/* expand the sub jobnet */
		rc = load_sub_jobnet_definition(inner_jobnet_id, run_type);
		if (rc != SUCCEED)
		{
			DBrollback();

			if (no_valid_flag != 0)
			{
				DBbegin();

				time(&now);
				tm = localtime(&now);
				strftime(now_time, sizeof(now_time), "%Y%m%d%H%M%S", tm);

				rc = DBexecute("update ja_run_jobnet_table set status = %d"
						" where inner_jobnet_id = %s",
						 JA_JOBNET_STATUS_ENDERR, row[0]);

				if (rc <= ZBX_DB_OK)
				{
					zbx_snprintf(msgwork, sizeof(msgwork), "%s", row[0]);
					ja_log("JALOADER200003", inner_jobnet_id, NULL, "ja_run_jobnet_table", msgwork);
					DBrollback();
					continue;
				}

				rc = DBexecute("insert into ja_run_jobnet_summary_table ("
						" inner_jobnet_id, update_date, invo_flag, run_type, status, job_status, jobnet_abort_flag,"
						" scheduled_time, start_time, end_time, public_flag, jobnet_id, user_name, jobnet_name, memo)"
						" values (%s, %s, 1, %s, %d, 2, 0,"
						" %s, %s, %s, %s, '%s', '%s', '%s', '%s')",
						row[0], row[2], row[3], JA_JOBNET_STATUS_ENDERR,
						row[4], now_time, now_time, row[5], row[6], row[7], row[8], memo);

				if (rc <= ZBX_DB_OK)
				{
					zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row[0], row[6]);
					ja_log("JALOADER200002", inner_jobnet_id, NULL, "ja_run_jobnet_summary_table", msgwork);
					DBrollback();
					continue;
				}
				DBcommit();
			}
			continue;
		}

		rc = DBexecute("insert into ja_run_jobnet_summary_table ("
				" inner_jobnet_id, update_date, invo_flag, run_type, status, job_status, jobnet_abort_flag,"
				" scheduled_time, start_time, end_time, public_flag, jobnet_id, user_name, jobnet_name, memo)"
				" values (%s, %s, 1, %s, %d, 0, 0,"
				" %s, 0, 0, %s, '%s', '%s', '%s', '%s')",
				row[0], row[2], row[3], JA_JOBNET_STATUS_BEGIN,
				row[4], row[5], row[6], row[7], row[8], memo);

		if (rc <= ZBX_DB_OK)
		{
			zbx_snprintf(msgwork, sizeof(msgwork), "%s %s", row[0], row[6]);
			ja_log("JALOADER200002", inner_jobnet_id, NULL, "ja_run_jobnet_summary_table", msgwork);
			DBrollback();
			continue;
		}

		DBcommit();
	}
	DBfree_result(result);

	return SUCCEED;
}

/******************************************************************************
 *                                                                            *
 * Function: main_jaloader_loop (main process)                                *
 *                                                                            *
 * Purpose: expand the jobnet that is registered in the schedule              *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
void	main_jaloader_loop()
{
	int	load_span;
	double	sec;

	zabbix_log(LOG_LEVEL_DEBUG, "In main_jaloader_loop() process_type:'%s' process_num:%d",
			ja_get_process_type_string(process_type), process_num);

	zbx_setproctitle("%s [connecting to the database]", ja_get_process_type_string(process_type));

	DBconnect(ZBX_DB_CONNECT_NORMAL);

	load_span = get_load_span();

	for (;;)
	{
		zbx_setproctitle("jobnet loader [performing loader]");

		/* scheduled start */
		sec = zbx_time();
		jobnet_load_schedule(load_span);
		sec = zbx_time() - sec;

		zabbix_log(LOG_LEVEL_DEBUG, "%s #%d (scheduled start) spent " ZBX_FS_DBL " seconds while processing rules",
				ja_get_process_type_string(process_type), process_num, sec);

		/* immediate start */
		sec = zbx_time();
		jobnet_load_immediate();
		sec = zbx_time() - sec;

		zabbix_log(LOG_LEVEL_DEBUG, "%s #%d (immediate start) spent " ZBX_FS_DBL " seconds while processing rules",
				ja_get_process_type_string(process_type), process_num, sec);

		ja_sleep_loop(CONFIG_JALOADER_INTERVAL);
	}
}
